@@ -31,13 +31,24 @@
 
 #include <stdint.h>
 
-#if defined(__APPLE__)
-#  include <libkern/OSByteOrder.h>
-#  define le64toh(x) OSSwapLittleToHostInt64(x)
-#elif defined(__FreeBSD__)
-#  include <sys/endian.h>
-#else
-#  include <endian.h>
+#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
+#   define le64toh(x) ((uint64_t)(x))
+#elif defined(__GNUC__) || defined(__clang__)
+#   ifdef __has_builtin && __has_builtin(__builtin_bswap64)
+#       define le64toh(x) __builtin_bswap64(x)
+#   endif
+#endif
+
+#ifndef le64toh
+#   define le646oh(x)                                  \
+    (((uint64_t)(x) << 56) |                           \
+     (((uint64_t)(x) << 40) & 0X00FF000000000000ULL) | \
+     (((uint64_t)(x) << 24) & 0X0000FF0000000000ULL) | \
+     (((uint64_t)(x) << 8)  & 0X000000FF00000000ULL) | \
+     (((uint64_t)(x) >> 8)  & 0X00000000FF000000ULL) | \
+     (((uint64_t)(x) >> 24) & 0X0000000000FF0000ULL) | \
+     (((uint64_t)(x) >> 40) & 0X000000000000FF00ULL) | \
+     ((uint64_t)(x)  >> 56))
 #endif
 
 #define ROTATE(x, b) (uint64_t)( ((x) << (b)) | ( (x) >> (64 - (b))) )
